@@ -217,7 +217,8 @@ amend_newdata <- function(newdata, fit, re_formula = NULL,
                           check_response = FALSE,
                           only_response = FALSE,
                           incl_autocor = TRUE,
-                          return_standata = TRUE) {
+                          return_standata = TRUE,
+                          new_objects = list()) {
   # amend newdata passed to predict and fitted methods
   # Args:
   #   newdata: a data.frame containing new data for prediction 
@@ -229,6 +230,7 @@ amend_newdata <- function(newdata, fit, re_formula = NULL,
   #   incl_autocor: Check data of autocorrelation terms?
   #   return_standata: Compute the data to be passed to Stan
   #                    or just return the updated newdata?
+  #   new_objects: see function 'add_new_objects'
   # Returns:
   #   updated data.frame being compatible with formula(fit)
   fit <- remove_autocor(fit, incl_autocor)
@@ -290,79 +292,73 @@ amend_newdata <- function(newdata, fit, re_formula = NULL,
     }
   }
   newdata <- combine_groups(newdata, group_vars)
-  # try to validate factor levels in newdata
-  if (is.data.frame(fit$data)) {
-    # validating is possible (implies brms > 0.5.0)
-    list_data <- lapply(as.list(fit$data), function(x)
-      if (is.numeric(x)) x else as.factor(x))
-    is_factor <- sapply(list_data, is.factor)
-    all_group_vars <- ulapply(fit$ranef$gcall, "[[", "groups")
-    dont_check <- c(all_group_vars, cens_vars)
-    dont_check <- names(list_data) %in% dont_check
-    factors <- list_data[is_factor & !dont_check]
-    if (length(factors)) {
-      factor_names <- names(factors)
-      factor_levels <- lapply(factors, levels) 
-      for (i in seq_along(factors)) {
-        new_factor <- newdata[[factor_names[i]]]
-        if (!is.null(new_factor)) {
-          if (!is.factor(new_factor)) {
-            new_factor <- factor(new_factor)
-          }
-          new_levels <- levels(new_factor)
-          old_levels <- factor_levels[[i]]
-          old_contrasts <- contrasts(factors[[i]])
-          to_zero <- is.na(new_factor) | new_factor %in% "zero__"
-          # don't add the 'zero__' level to response variables
-          is_resp <- factor_names[i] %in% all.vars(bterms$respform)
-          if (!is_resp && any(to_zero)) {
-            levels(new_factor) <- c(new_levels, "zero__")
-            new_factor[to_zero] <- "zero__"
-            old_levels <- c(old_levels, "zero__")
-            old_contrasts <- rbind(old_contrasts, zero__ = 0)
-          }
-          if (any(!new_levels %in% old_levels)) {
-            stop2("New factor levels are not allowed.",
-                  "\nLevels found: ", paste(new_levels, collapse = ", ") ,
-                  "\nLevels allowed: ", paste(old_levels, collapse = ", "))
-          }
-          newdata[[factor_names[i]]] <- factor(new_factor, old_levels)
-          # don't use contrasts(.) here to avoid dimension checks
-          attr(newdata[[factor_names[i]]], "contrasts") <- old_contrasts
+  # validate factor levels in newdata
+  list_data <- lapply(as.list(fit$data), function(x)
+    if (is_like_factor(x)) as.factor(x) else x)
+  is_factor <- sapply(list_data, is.factor)
+  all_group_vars <- ulapply(fit$ranef$gcall, "[[", "groups")
+  dont_check <- c(all_group_vars, cens_vars)
+  dont_check <- names(list_data) %in% dont_check
+  factors <- list_data[is_factor & !dont_check]
+  if (length(factors)) {
+    factor_names <- names(factors)
+    factor_levels <- lapply(factors, levels) 
+    for (i in seq_along(factors)) {
+      new_factor <- newdata[[factor_names[i]]]
+      if (!is.null(new_factor)) {
+        if (!is.factor(new_factor)) {
+          new_factor <- factor(new_factor)
         }
+        new_levels <- levels(new_factor)
+        old_levels <- factor_levels[[i]]
+        old_contrasts <- contrasts(factors[[i]])
+        to_zero <- is.na(new_factor) | new_factor %in% "zero__"
+        # don't add the 'zero__' level to response variables
+        is_resp <- factor_names[i] %in% all.vars(bterms$respform)
+        if (!is_resp && any(to_zero)) {
+          levels(new_factor) <- c(new_levels, "zero__")
+          new_factor[to_zero] <- "zero__"
+          old_levels <- c(old_levels, "zero__")
+          old_contrasts <- rbind(old_contrasts, zero__ = 0)
+        }
+        if (any(!new_levels %in% old_levels)) {
+          stop2("New factor levels are not allowed.",
+                "\nLevels found: ", paste(new_levels, collapse = ", ") ,
+                "\nLevels allowed: ", paste(old_levels, collapse = ", "))
+        }
+        newdata[[factor_names[i]]] <- factor(new_factor, old_levels)
+        # don't use contrasts(.) here to avoid dimension checks
+        attr(newdata[[factor_names[i]]], "contrasts") <- old_contrasts
       }
     }
-    # validate monotonic variables
-    if (is.formula(bterms$mo)) {
-      take_num <- !is_factor & names(list_data) %in% all.vars(bterms$mo)
-      # factors have already been checked
-      num_mo_vars <- names(list_data)[take_num]
-      for (v in num_mo_vars) {
-        # use 'get' to check whether v is defined in newdata
-        new_values <- get(v, newdata)
-        min_value <- min(list_data[[v]])
-        invalid <- new_values < min_value | 
-                   new_values > max(list_data[[v]]) |
-                   !is_wholenumber(new_values)
-        if (sum(invalid)) {
-          stop2("Invalid values in variable '", v, "': ",
-                paste(new_values[invalid], collapse = ","))
-        }
-        attr(newdata[[v]], "min") <- min_value
+  }
+  # validate monotonic variables
+  if (is.formula(bterms$mo)) {
+    take_num <- !is_factor & names(list_data) %in% all.vars(bterms$mo)
+    # factors have already been checked
+    num_mo_vars <- names(list_data)[take_num]
+    for (v in num_mo_vars) {
+      # use 'get' to check whether v is defined in newdata
+      new_values <- get(v, newdata)
+      min_value <- min(list_data[[v]])
+      invalid <- new_values < min_value | 
+                 new_values > max(list_data[[v]]) |
+                 !is_wholenumber(new_values)
+      if (sum(invalid)) {
+        stop2("Invalid values in variable '", v, "': ",
+              paste(new_values[invalid], collapse = ","))
       }
+      attr(newdata[[v]], "min") <- min_value
     }
-    # brms:::update_data expects all original variables to be present
-    # even if not actually used later on
-    rsv_vars <- rsv_vars(bterms)
-    used_vars <- unique(c(names(newdata), all.vars(bterms$allvars), rsv_vars))
-    all_vars <- all.vars(str2formula(names(model.frame(fit))))
-    unused_vars <- setdiff(all_vars, used_vars)
-    if (length(unused_vars)) {
-      newdata[, unused_vars] <- NA
-    }
-  } else {
-    warning2("Validity of factors cannot be checked for ", 
-             "fitted model objects created with brms <= 0.5.0.")
+  }
+  # brms:::update_data expects all original variables to be present
+  # even if not actually used later on
+  rsv_vars <- rsv_vars(bterms)
+  used_vars <- unique(c(names(newdata), all.vars(bterms$allvars), rsv_vars))
+  all_vars <- all.vars(str2formula(names(model.frame(fit))))
+  unused_vars <- setdiff(all_vars, used_vars)
+  if (length(unused_vars)) {
+    newdata[, unused_vars] <- NA
   }
   # validate grouping factors
   gnames <- unique(new_ranef$group)
@@ -378,6 +374,7 @@ amend_newdata <- function(newdata, fit, re_formula = NULL,
     }
   }
   if (return_standata) {
+    fit <- add_new_objects(fit, newdata, new_objects)
     control <- list(
       is_newdata = TRUE, not4stan = TRUE, 
       old_levels = old_levels, save_order = TRUE, 
@@ -385,12 +382,13 @@ amend_newdata <- function(newdata, fit, re_formula = NULL,
       only_response = only_response,
       old_cat = is_old_categorical(fit)
     )
+    # ensure correct handling of functions like poly or scale
     old_terms <- attr(model.frame(fit), "terms")
     terms_attr <- c("variables", "predvars")
     control$terms_attr <- attributes(old_terms)[terms_attr]
+    # some components should not be computed based on newdata
     has_mo <- length(get_effect(bterms, "mo")) > 0L
     if (has_trials(fit$family) || has_cat(fit$family) || has_mo) {
-      # some components should not be computed based on newdata
       pars <- c(names(bterms$nlpars), intersect(auxpars(), names(bterms)))
       comp <- c("trials", "ncat", paste0("Jmo", c("", paste0("_", pars))))
       old_standata <- rmNULL(standata(fit)[comp])
@@ -399,12 +397,14 @@ amend_newdata <- function(newdata, fit, re_formula = NULL,
       names(Jmo) <- sub("^Jmo$", "mu", sub("^Jmo_", "", names(Jmo)))
       control[["Jmo"]] <- Jmo 
     }
+    if (is.cor_car(fit$autocor)) {
+      if (isTRUE(nzchar(bterms$time$group))) {
+        old_loc_data <- get(bterms$time$group, fit$data)
+        control$old_locations <- levels(factor(old_loc_data))
+      }
+    }
     control$smooths <- make_smooth_list(bterms, model.frame(fit))
     control$gps <- make_gp_list(bterms, model.frame(fit))
-    if (is(fit$autocor, "cor_fixed")) {
-      median_V <- median(diag(fit$autocor$V), na.rm = TRUE)
-      fit$autocor$V <- diag(median_V, nrow(newdata))
-    }
     knots <- attr(model.frame(fit), "knots")
     newdata <- make_standata(
       new_formula, data = newdata, family = fit$family, 
@@ -414,6 +414,37 @@ amend_newdata <- function(newdata, fit, re_formula = NULL,
   newdata
 }
 
+add_new_objects <- function(x, newdata, new_objects = list()) {
+  # allows for updating of objects containing new data
+  # which cannot be passed via argument 'newdata'
+  # Args:
+  #   x: object of class 'brmsfit'
+  #   new_objects: optional list of new objects
+  # Return:
+  #   a possibly updated 'brmsfit' object
+  stopifnot(is.brmsfit(x), is.data.frame(newdata))
+  if (is.cor_sar(x$autocor)) {
+    if ("W" %in% names(new_objects)) {
+      x$autocor <- cor_sar(new_objects$W, type = x$autocor$type)
+    } else {
+      message("Using the identity matrix as weighting matrix by default")
+      x$autocor$W <- diag(nrow(newdata))
+    }
+  }
+  # do not include cor_car here as the adjacency matrix
+  # (or subsets of it) should be the same for newdata 
+  if (is.cor_fixed(x$autocor)) {
+    if ("V" %in% names(new_objects)) {
+      x$autocor <- cor_fixed(new_objects$V)
+    } else {
+      message("Using the median variance by default")
+      median_V <- median(diag(x$autocor$V), na.rm = TRUE)
+      x$autocor$V <- diag(median_V, nrow(newdata)) 
+    }
+  }
+  x
+}
+
 get_model_matrix <- function(formula, data = environment(formula),
                              cols2remove = NULL, rename = TRUE, ...) {
   # Construct Design Matrices for \code{brms} models
