@@ -2,11 +2,13 @@
 extract_draws.brmsfit <- function(x, newdata = NULL, re_formula = NULL, 
                                   allow_new_levels = FALSE, 
                                   sample_new_levels = "uncertainty",
-                                  incl_autocor = TRUE, subset = NULL, 
-                                  nsamples = NULL, nug = NULL, ...) {
+                                  new_objects = list(), incl_autocor = TRUE, 
+                                  subset = NULL, nsamples = NULL, 
+                                  nug = NULL, ...) {
   # extract all data and posterior draws required in (non)linear_predictor
   # Args:
   #   see doc of logLik.brmsfit
+  #   ...: passed to amend_newdata
   # Returns:
   #   A named list to be interpreted by linear_predictor
   snl_options <- c("uncertainty", "gaussian", "old_levels")
@@ -15,7 +17,9 @@ extract_draws.brmsfit <- function(x, newdata = NULL, re_formula = NULL,
   bterms <- parse_bf(formula(x), family = family(x))
   subset <- subset_samples(x, subset, nsamples)
   nsamples <- nsamples(x, subset = subset)
-  newd_args <- nlist(fit = x, newdata, re_formula, allow_new_levels)
+  newd_args <- nlist(
+    fit = x, newdata, re_formula, allow_new_levels, new_objects
+  )
   draws <- nlist(
     f = prepare_family(x), nsamples = nsamples,
     data = do.call(amend_newdata, c(newd_args, list(...)))
@@ -83,11 +87,13 @@ extract_draws.brmsfit <- function(x, newdata = NULL, re_formula = NULL,
     draws$rescor <- do.call(as.matrix, c(am_args, pars = "^rescor_"))
     draws$Sigma <- get_cov_matrix(sd = draws$sigma, cor = draws$rescor)$cov
   }
-  if (use_cov(x$autocor)) {
+  if (use_cov(x$autocor) || is.cor_sar(x$autocor)) {
     # only include autocor samples on the top-level of draws 
-    # when using the covariance formulation of ARMA structures
+    # when using the covariance formulation of ARMA / SAR structures
     draws <- c(draws, 
-      extract_draws_autocor(x, newdata = newdata, subset = subset)
+      extract_draws_autocor(
+        x, subset, newdata = newdata, sdata = draws$data
+      )
     )
   }
   draws
@@ -119,9 +125,9 @@ extract_draws.btnl <- function(x, C, nlpar = "", ...) {
 #' @export
 extract_draws.btl <- function(x, fit, newdata = NULL, re_formula = NULL, 
                               allow_new_levels = FALSE, 
-                              sample_new_levels = FALSE,
-                              incl_autocor = TRUE, subset = NULL, 
-                              nlpar = "", smooths_only = FALSE, 
+                              sample_new_levels = "uncertainty",
+                              new_objects = list(), incl_autocor = TRUE, 
+                              subset = NULL, nlpar = "", smooths_only = FALSE, 
                               mv = FALSE, nug = NULL, ...) {
   # extract draws of all kinds of effects
   # Args:
@@ -154,10 +160,13 @@ extract_draws.btl <- function(x, fit, newdata = NULL, re_formula = NULL,
   usc_nlpar <- usc(usc(nlpar))
   newd_args <- nlist(
     fit, newdata, re_formula, allow_new_levels, 
-    check_response = FALSE
+    new_objects, check_response = FALSE
+  )
+  draws <- list(
+    # supresses messages of add_new_objects
+    data = suppressMessages(do.call(amend_newdata, newd_args)), 
+    old_cat = is_old_categorical(fit)
   )
-  draws <- list(data = do.call(amend_newdata, newd_args), 
-                old_cat = is_old_categorical(fit))
   draws[names(dots)] <- dots
   
   if (smooths_only) {
@@ -194,7 +203,9 @@ extract_draws.btl <- function(x, fit, newdata = NULL, re_formula = NULL,
   if (!use_cov(fit$autocor) && (!nzchar(nlpar) || mv)) {
     # only include autocorrelation parameters in draws for mu
     draws <- c(draws, 
-      extract_draws_autocor(fit, newdata = newdata, subset = subset)
+      extract_draws_autocor(
+        fit, subset, newdata = newdata, sdata = draws$data
+      )
     )
   }
   structure(draws, predicted = TRUE)
@@ -602,9 +613,13 @@ extract_draws_re <- function(ranef, args, sdata, nlpar = "",
   draws
 }
 
-extract_draws_autocor <- function(fit, newdata = NULL, subset = NULL) {
+extract_draws_autocor <- function(fit, subset = NULL,
+                                  newdata = NULL, sdata = NULL) {
   # extract draws of autocorrelation parameters
+  # Args:
+  #   sdata: output of make_standata
   stopifnot(is.brmsfit(fit))
+  is_newdata <- !is.null(newdata)
   args <- nlist(x = fit, subset)
   draws <- list()
   if (get_ar(fit$autocor)) {
@@ -616,12 +631,30 @@ extract_draws_autocor <- function(fit, newdata = NULL, subset = NULL) {
   if (get_arr(fit$autocor)) {
     draws[["arr"]] <- do.call(as.matrix, c(args, pars = "^arr\\["))
   }
+  if (is.cor_sar(fit$autocor)) {
+    draws[["lagsar"]] <- do.call(as.matrix, c(args, pars = "^lagsar$"))
+    draws[["errorsar"]] <- do.call(as.matrix, c(args, pars = "^errorsar$"))
+  }
+  if (is.cor_car(fit$autocor)) {
+    if (is_newdata) {
+      group <- parse_time(fit$autocor)$group
+      if (!isTRUE(nzchar(group))) {
+        stop2("Without a grouping factor, CAR models cannot handle newdata.")
+      }
+    }
+    gcar <- sdata[["Jloc"]]
+    Zcar <- matrix(rep(1, length(gcar)))
+    draws[["Zcar"]] <- prepare_Z(Zcar, list(gcar))
+    rcar <- do.call(as.matrix, c(args, pars = "^rcar\\["))
+    rcar <- rcar[, unique(gcar), drop = FALSE]
+    draws[["rcar"]] <- rcar
+  }
   if (is(fit$autocor, "cor_bsts")) {
-    if (is.null(newdata)) {
-      draws[["loclev"]] <- do.call(as.matrix, c(args, pars = "^loclev\\["))
-    } else {
+    if (is_newdata) {
       warning2("Local level terms are currently ignored ", 
                "when 'newdata' is specified.")
+    } else {
+      draws[["loclev"]] <- do.call(as.matrix, c(args, pars = "^loclev\\["))
     }
   }
   draws
