@@ -70,7 +70,6 @@ rename_pars <- function(x) {
   # perform the actual renaming in x$fit@sim
   x <- do_renaming(x, change)
   x <- compute_quantities(x)
-  x$fit@sim$pars_oi <- names(x$fit@sim$dims_oi)
   x
 }
 
@@ -780,6 +779,7 @@ do_renaming <- function(x, change) {
   for (i in seq_along(change)) {
     x <- .do_renaming(x, change[[i]])
   }
+  x$fit@sim$pars_oi <- names(x$fit@sim$dims_oi)
   x
 }
 
@@ -788,10 +788,11 @@ reorder_pars <- function(x) {
   # Args:
   #   x: brmsfit object
   all_classes <- c(
-    "b", "bmo", "bcs", "bme", "ar", "ma", "arr", "sd", "cor", 
-    "sds", "sdgp", "lscale", auxpars(), "temp", "sigmaLL", 
-    "rescor", "delta", "lasso", "simplex", "r", "s", "zgp", 
-    "loclev", "Xme", "prior", "lp"
+    "b", "bmo", "bcs", "bme", "ar", "ma", "arr", "lagsar",
+    "errorsar", "car", "sdcar", "sigmaLL", "sd", "cor", "sds", 
+    "sdgp", "lscale", auxpars(), "temp", "rescor", "delta", 
+    "lasso", "simplex", "r", "s", "zgp", "rcar", "loclev", 
+    "Xme", "prior", "lp"
   )
   # reorder parameter classes
   class <- get_matches("^[^[:digit:]_]+", x$fit@sim$pars_oi)
