@@ -10,6 +10,8 @@ test_that("brm produces expected errors", {
                "No non-linear parameters specified")
   expect_error(brm(bf(y ~ a, a ~ 1, nl = TRUE), family = acat()),
                "Non-linear formulas are not yet allowed for this family")
+  expect_error(brm(bf(y ~ 0 + a), dat, family = cumulative()),
+               "Cannot remove the intercept in an ordinal model")
   expect_error(brm(y ~ mono(1), dat),
                "No variable supplied to function 'mo'")
   expect_error(brm(bf(y | se(sei) ~ x, sigma ~ x), dat),
@@ -36,7 +38,7 @@ test_that("brm produces expected errors", {
   expect_error(brm(y ~ 1, dat, autocor = cor_ar(~x+y|g)), 
                "Autocorrelation structures may only contain 1 time variable")
   expect_error(brm(y ~ 1, dat, autocor = cor_ar(x~t1|g1)), 
-               "Autocorrelation formula must be one-sided")
+               "Autocorrelation formulas must be one-sided")
   expect_error(brm(y ~ 1, dat, autocor = cor_ar(~1|g1/g2)), 
                paste("Illegal grouping term: g1/g2"))
   expect_error(brm(y ~ 1, dat, poisson(), autocor = cor_ma(~x)),
