@@ -1,14 +1,24 @@
 # most tests of prior related stuff can be found in tests.make_stancode.R
 
 test_that("get_prior finds all classes for which priors can be specified", {
-  expect_equal(sort(get_prior(count ~ log_Base4_c * Trt_c 
-                              + (1|patient) + (1+Trt_c|visit),
-                              data = epilepsy, family = "poisson")$class),
-               sort(c(rep("b", 5), c("cor", "cor"), "Intercept", rep("sd", 6))))
-  expect_equal(sort(get_prior(rating ~ treat + period + cse(carry),
-                         data = inhaler, family = "sratio", 
-                         threshold = "equidistant")$class),
-               sort(c(rep("b", 5), "delta", "Intercept")))
+  expect_equal(
+    sort(
+      get_prior(
+        count ~ log_Base4_c * Trt_c + (1|patient) + (1+Trt_c|visit),
+        data = epilepsy, family = "poisson"
+      )$class
+    ),
+    sort(c(rep("b", 5), c("cor", "cor"), "Intercept", rep("sd", 6)))
+  )
+  expect_equal(
+    sort(
+      get_prior(
+        rating ~ treat + period + cse(carry), data = inhaler, 
+        family = sratio(threshold = "equidistant")
+      )$class
+    ),
+    sort(c(rep("b", 5), "delta", "Intercept"))
+  )
 })
 
 test_that("print for class brmsprior works correctly", {
